/* Copyright Andy Mitchell 2007, 2008. No warranties */

function AccordionUI(gtdinboxPage, elParent){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this.sections = [];
	this._currentSection = "";
	this._eventObservers = new GtdEventObservers(this);
	
	this.el = this._doc.createElement("DIV");
		this.el.className = "accordion";
	if( elParent ) elParent.appendChild( this.el );
	
	this._eventObservers.observe( this.el, "click", this._clickAccordion, false);
	
}

AccordionUI.prototype.addGroupTitle = function(title){
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = title;
		elTitle.className = "group-title";
	this.el.appendChild( elTitle );
};

AccordionUI.prototype.addSection = function(title, elContent){
try{
	var elTitle = this._doc.createElement("DIV");
		elTitle.innerHTML = "<span>" + title + "</span>";
		elTitle.className = "title";
	if( !elContent ) elContent = this._doc.createElement("DIV");
	GIPROTOTYPE.addClassName( elContent, "content" );
	
	var section = {'title':title, 'elTitle':elTitle, 'elTitleSpan':elTitle.firstChild, 'elContent':elContent};
	this.el.appendChild( elTitle );
	this.el.appendChild( elContent );
	this.sections.push( section );
	
	// refresh list
	if( !this._currentSection ) this._currentSection = title;
	this.currentSection( this._currentSection );
	
	return section;
}catch(e){
	if( GTDDEBUGMODE ) alert("AccordionUI addSection\n" + e.toString());
}
};

AccordionUI.prototype.currentSection = function(v){
try{
	if( typeof v=="undefined" ){
		this._currentSection;
		for( var i = 0; i < this.sections.length; i++ ){
			if( this.sections[i].title==this._currentSection ){
				return this.sections[i];
			}
		}
		return;
	}
	
	this._currentSection = v;
	
	for( var i = 0; i < this.sections.length; i++ ){
		if( !this._currentSection ){	// Just hide all
			GIPROTOTYPE.removeClassName( this.sections[i].elTitle, "selected" );
			this.sections[i].elContent.style.display = "none";
		}else{
			if( !this.sections[i].hidden && this.sections[i].title==v ){
				this.sections[i].elContent.style.display = "";
				GIPROTOTYPE.addClassName( this.sections[i].elTitle, "selected" );
			}else{
				GIPROTOTYPE.removeClassName( this.sections[i].elTitle, "selected" );
				this.sections[i].elContent.style.display = "none";
			}
		}
	}
	
}catch(e){
	if( GTDDEBUGMODE ) alert("AccordionUI currentSection\n" + e.toString());
}
};

AccordionUI.prototype.disableSection = function(v, disable){
	for( var i = 0; i < this.sections.length; i++ ){
		if( this.sections[i].title==v ){	// Found it
			if( disable ){
				GIPROTOTYPE.addClassName( this.sections[i].elTitle, "disabled" );
				GIPROTOTYPE.removeClassName( this.sections[i].elTitle, "selected" );
				this.sections[i].elContent.style.display = "none";
			}else{
				GIPROTOTYPE.removeClassName( this.sections[i].elTitle, "disabled" );
				this.currentIsFirstNonDisabled();
			}
		}
	}
};
AccordionUI.prototype.hideSection = function(v, hide){
	for( var i = 0; i < this.sections.length; i++ ){
		if( this.sections[i].title==v ){	// Found it
			this.sections[i].hidden = hide;
			if( hide ){
				GIPROTOTYPE.removeClassName( this.sections[i].elTitle, "selected" );
				this.sections[i].elTitle.style.display = "none";
				this.sections[i].elContent.style.display = "none";
			}else{
				this.sections[i].elTitle.style.display = "";
				this.currentIsFirstNonDisabled();
			}
		}
	}
};

AccordionUI.prototype.currentIsFirstNonDisabled = function(){
	for( var i = 0; i < this.sections.length; i++ ){
		if( !this.sections[i].hidden && !GIPROTOTYPE.hasClassName( this.sections[i].elTitle, "disabled" ) ){
			this.currentSection( this.sections[i].title );
			break;
		}
	}
};


AccordionUI.prototype._clickAccordion = function(event){
try{
	var el = GIPROTOTYPE.element(event);
	
	if( GIPROTOTYPE.hasClassName(el, "disabled") ) return;
	
	for( var i = 0; i < this.sections.length; i++ ){
		if( this.sections[i].elTitle==el || (this.sections[i].elTitleSpan==el && !el.hasAttribute("gtdSearch")) ){	// Found the click
			if( this.currentSection()==this.sections[i] ){	// Current choice; just hide it
				this.currentSection(false);
			}else{
				this.currentSection( this.sections[i].title );
			}
			break;
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("AccordionUI _clickAccordion\n" + e.toString());
}
};

AccordionUI.prototype.check

AccordionUI.prototype.destroy = function(){
	
	if( !this._gtdinboxPage ) return;
	
	this._eventObservers.destroy();
	this._gtdinboxPage = null;
	
};