/* Copyright Andy Mitchell 2007, 2008. No warranties */

function LabelData(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	
	this.el = null;
	this.leafName = null;
	this.labelName = null;
	this.type = null;
	this.depth = 0;
	this.isPlaceHolder = true;
	this.parent = null;
	this.children = {};
	this.childrenCount = 0;
	this._clone = null;
}

LabelData.prototype.init = function(type, leafName, labelName, depth, parent){
	this.type = type;
	this.leafName = leafName;
	this.labelName = labelName;
	this.depth = depth;
	if( parent ) parent.addChildLabel( this );
};

LabelData.prototype.addChildLabel = function(label){
	label.parent = this;
	if( !this.children[label.labelName] ) this.childrenCount++;
	this.children[label.labelName] = label;
};

LabelData.prototype.selectLabels = function(labelnames){
	if( labelnames instanceof Array ) labelnames = GIPROTOTYPE.arrayToHash(labelnames);
	this.applyToChildren( GIPROTOTYPE.bind( this, function(label){
			if( labelnames[label.labelName] ){	// It should be selected
				GIPROTOTYPE.addClassName( label.elName, "selected" );
			}else{
				GIPROTOTYPE.removeClassName( label.elName, "selected" );
			}
		}), true, true);
};

LabelData.prototype.openLabel = function(){
	if( this.elExpander ){
		this.elExpander.src = "chrome://gtdinbox/content/collapsetree-open.png";
		this.elExpander.alt = "-";
		this.applyToChildren( this._openLabel, false, true );
	}
};
LabelData.prototype._openLabel = function(label){
	label.el.style.display = "";
};

LabelData.prototype.openLabelsToHere = function(){
	// Open all parents to make sure this is open; but no further
	
	var label = this;
	while( label.parent ){
		label.parent.applyToChildren( this._openLabelsToHere, false, true );	// Applies to children; but does not recurse.
		label = label.parent;
	}
};
LabelData.prototype._openLabelsToHere = function(label){
	label.el.style.display = "";
	if( label.elExpander ){
		label.elExpander.src = "chrome://gtdinbox/content/collapsetree-open.png";
		label.elExpander.alt = "-";
	}
};

LabelData.prototype.closeLabel = function(){
	if( this.elExpander ){
		this.elExpander.src = "chrome://gtdinbox/content/collapsetree-closed.png";
		this.elExpander.alt = "+";
	}
	this.applyToChildren( this._closeLabel, true);
};
LabelData.prototype._closeLabel = function(label){
	label.el.style.display = "none";
	if( label.elExpander ){
		label.elExpander.src = "chrome://gtdinbox/content/collapsetree-closed.png";
		label.elExpander.alt = "+";
	}
};

LabelData.prototype.applyToParents = function(f){
	var label = this;
	while( label.parent ){
		f(label.parent);
		label = label.parent;
	}
};

LabelData.prototype.applyToChildren = function(f, recurse, includeThis){
	if( includeThis ){
		if( f(this) ) return;	// If f returns true; break any recursion.
	}
	for( c in this.children ){
		f( this.children[c] );
		if( recurse ){
			if( this.children[c].applyToChildren(f,recurse) ) return;
		}
	}
};

LabelData.prototype.getFirstChild = function(){
	for( c in this.children ){
		return this.children[c];
	}
};

LabelData.prototype.transformToIncludeExpander = function(){	
	var els = this.el.getElementsByTagName("IMG");
	for( var i = 0; i < els.length; i++ ) if( els[i].className=="label-folder-toggle" ) return;
		
	this.elExpander = this._doc.createElement("IMG");
		this.elExpander.setAttribute("gtdLabelFullName", this.labelName);
		this.elExpander.className = "label-folder-toggle";
		
	this.elExpander.src = "chrome://gtdinbox/content/collapsetree-closed.png";
	this.elExpander.alt = "+";
	
	
	var elLabelDiv = GIPROTOTYPE.getElementsByClassName( this.el, "yyT6sf" )[0];
	if( elLabelDiv ){
		elLabelDiv.insertBefore( this.elExpander, elLabelDiv.firstChild );
	}else{
		this.elName.parentNode.insertBefore( this.elExpander, this.elName );
	}
};

LabelData.prototype.transformToIndent = function(){	
	this.elName.parentNode.style.paddingLeft = (this.depth*12) + "px";
	
};

LabelData.prototype.clone = function(typeEl, typeLabelsHash){
	var label = new LabelData(this._gtdinboxPage);
		label.init( this.type, this.leafName, this.labelName, this.depth );
		label.isPlaceHolder = this.isPlaceHolder;
	
	label.el = this._doc.createElement("DIV");
		label.el.className = label.isPlaceHolder? "placeholder-label" : "label";
		label.el.setAttribute("gtdName", label.leafName);
		label.el.setAttribute("gtdType", label.type);
		if( !this.isPlaceHolder ) label.el.setAttribute("gtdLabelName", label.labelName);
		label.el.setAttribute("gtdLabelFullName", label.labelName);
		if( this.parent ) label.el.setAttribute("gtdLabelParentName", this.parent.labelName );
		
	label.elName = this._doc.createTextNode(this.leafName);
		label.el.appendChild( label.elName );
		
	if( this.depth > 0 ){
		label.transformToIndent();
		label.el.style.display = "none";	// Hide this child
	}
	if( this.childrenCount > 0 ){
		label.transformToIncludeExpander();
	}
	
	typeEl.appendChild( label.el );
	typeLabelsHash[label.labelName] = label;
	
	for( c in this.children ){
		label.addChildLabel( this.children[c].clone(typeEl, typeLabelsHash) );
	}
	
	return label;
};

LabelData.prototype.destroy = function(){
	this.el = null;
	if( this.children ) delete this.children;
};