/* Copyright Andy Mitchell 2007, 2008. No warranties */

function LabelSelectUI(gtdinboxPage, elParent, createWithAjax, threadActions, hideNewLabel, restrictTypes ){
	this.gtdinboxPage = gtdinboxPage;
	this.doc = gtdinboxPage.doc;
	this.settings = gtdinboxPage.settings;
	this._labels = gtdinboxPage._labels;
	this.GtdEventObservers = new GtdEventObservers(this);
	this.eventHandlers = new GtdEventHandlers_Mutator(this);
	//this._showHiders = [];
	//this._inlineSelectors = [];
	this._optionsShortList = {maxSelected:100, maxBody:100, maxRecentByName:3, maxRecent:3};
	this._createWithAjax = createWithAjax;
	this._threadActions = threadActions;
	this._hideNewLabel = hideNewLabel || false;
	this._labelSelectors = {};
	this._alwaysRefresh = [];
	this._restrictTypes = restrictTypes;
	
	this._labelActiveHash = {};
	this._types = [];
	for( t in this.settings.labels.types ){
		if( t!="Old" ) this._types.push(t);
	}

	this.create(elParent);
	
	//this._labels.addEventHandler( this, "refresh-labels", this.refresh );
	this._labels.addEventHandler( this, "refresh-labels", function(a, labelsChange){
		if( this.thread && labelsChange.changed ){
			this.thread.refreshData( GIPROTOTYPE.bind(this, function(){
				this.refresh();
			}));
		}else{
			this.refresh();
		}
	} );
	
}


LabelSelectUI.prototype.create = function(elParent){

	this.el = this.doc.createElement("DIV");
		this.el.className = "label-selector";
	if( elParent ) elParent.appendChild( this.el );
	
	
	var elLabelsContainer = this.doc.createElement("DIV");
		elLabelsContainer.className = "label-selector-labels-container";
	this.el.appendChild( elLabelsContainer );
	
	this.elInlineSelectors = this.doc.createElement("DIV");
		this.elInlineSelectors.className = "inline-selectors";
	elLabelsContainer.appendChild( this.elInlineSelectors );
	
	
	var r = this._createRow( this.gtdinboxPage.stringBundle.getString("LabelSelectUI.Titles.OtherCombos") );
	this.elCombos = r.elContainer;
		GIPROTOTYPE.addClassName( this.elCombos, "label-selector-othercombos");
		this.elCombosClear = this.doc.createElement("DIV");
			this.elCombosClear.style.clear = "both";
		this.elCombos.appendChild( this.elCombosClear );
	this.elCombosContainer = r.elRow;
	elLabelsContainer.appendChild( this.elCombosContainer );
	
	this.elActionsContainer = this.doc.createElement("DIV");
		this.elActionsContainer.className = "actions";
	this.el.appendChild( this.elActionsContainer );
	if( !this._hideNewLabel ){
		this.elCreate = this.doc.createElement("IMG");
			this.elCreate.src = "chrome://gtdinbox/content/add.png";
			this.elCreate.title = this.gtdinboxPage.stringBundle.getString("LabelSelectUI.NewLabel.Tooltip");
			this.elCreate.className = "create-label";
			this.GtdEventObservers.observe(this.elCreate, "click", this.clickCreate, false);
		this.elActionsContainer.appendChild( this.elCreate );
		
	}
	var elClear = this.doc.createElement("DIV");
		elClear.style.clear = "both";
	elLabelsContainer.appendChild( elClear );
	var elClear = this.doc.createElement("DIV");
		elClear.style.clear = "both";
	this.el.appendChild( elClear );
	
	this.GtdEventObservers.observe( this.el, "mousedown", this.clickLabel, false);
	
	
	
};

LabelSelectUI.prototype._createRow = function(title, elContent){
	var elRow = this.doc.createElement("DIV");
		elRow.className = "label-selector-row";
	
	var elTitle = this.doc.createElement("DIV");
		elTitle.className = "label-selector-title";
		elTitle.innerHTML = title;
	elRow.appendChild( elTitle );
	
	var elContainer = this.doc.createElement("DIV");
		elContainer.className = "label-selector-labels";
		if( elContent ) elContainer.appendChild( elContent );
	elRow.appendChild( elContainer );
	
	var elClear = this.doc.createElement("DIV");
		elClear.style.clear = "both";
	elRow.appendChild( elClear );
	
	return {'elRow':elRow, 'elTitle':elTitle, 'elContainer':elContainer};
	
};

LabelSelectUI.prototype.refresh = function(thread){
	try{
		if( thread ){
			this.thread = thread;
			this._optionsShortList.thread = thread;
		}
		
		// Rebuild UI
		if( this._lastRefresh && this._labels._lastRefresh && this._labels._lastRefresh<=this._lastRefresh ){	// No change to labels. Just remove any selections (and update Recent)
			
			for( var i = 0; i < this._alwaysRefresh.length; i++ ) this._alwaysRefresh[i].refresh(null, true);
			
		}else{
			
			
			this._alwaysRefresh.length = 0;
						
			for( var i = 0; i < this._types.length; i++ ){
				var t = this._types[i];
				if( this.gtdinboxPage._labels.typeBuckets[t] ){
					if( !this._restrictTypes || this._restrictTypes[t] ){
						if( !this._labelSelectors[t] ){	// Create
							if( !this.gtdinboxPage.settings.ui.labelSelectors[t] ) this.gtdinboxPage.settings.ui.labelSelectors[t] = {'dropdown':true};
							for( tt in this.gtdinboxPage.settings.ui.labelSelectors[t] ){
								if( this.gtdinboxPage.settings.ui.labelSelectors[t][tt] ){
									switch( tt ){
										case "inline":
											this._labelSelectors[t] = new LabelSelectUIInline(this.gtdinboxPage, true);
											this.elInlineSelectors.appendChild( this._createRow(this.gtdinboxPage.stringBundle.getStringSafe("LabelSelectUI.Titles." + t, t),this._labelSelectors[t].el).elRow );
											break;
										case "hybrid":
											if( GTDDEVMODE ){
												this._labelSelectors[t] = new LabelSelectUIHybrid(this.gtdinboxPage, true);
												this.elInlineSelectors.appendChild( this._createRow(this.gtdinboxPage.stringBundle.getStringSafe("LabelSelectUI.Titles." + t, t),this._labelSelectors[t].el).elRow );
												this._alwaysRefresh.push( this._labelSelectors[t] );
												break;
											}
										case "dropdown":
											this._labelSelectors[t] = new LabelSelectUIDropdown(this.gtdinboxPage);
											this.elCombos.insertBefore( this._labelSelectors[t].el, this.elCombosClear );
											break;
										
									}
								}
							}
						}
						this._labelSelectors[t].refresh(t);
					}
				}
			}
			
		}	
		

		
		this.elCombosContainer.style.display = ( this.elCombos.childNodes.length<=1 )? "none" : "";
		
		this._lastRefresh = (new Date()).valueOf();
	
	
		
		// Choose selected:
		if( this.thread ) this.selected( this.thread.labels );
		
		
	}catch(e){
		if( GTDDEBUGMODE ) alert("labelselectui refresh\n" + e.toString());
	}
};

/*
LabelSelectUI.prototype.getShowHiderByType = function(t){
	for( var i = 0; i < this._showHiders.length; i++ ){
		if( this._showHiders[i].labelTypeName==t ){
			return this._showHiders[i];
		}
	}
};
*/

LabelSelectUI.prototype.getLabelsByName = function(labelname){
	// Use XPath to get the label
	//var labelsList = this.doc.evaluate( "//div[@gtdviewid='" + this.gtdinboxPage.view("gtdviewid") + "']//div[starts-with(@class,'label-selector')]//span[@gtdlabelname='" + labelname + "'] | //div[@gtdviewid='" + this.gtdinboxPage.view("gtdviewid") + "']//div[starts-with(@class,'label-selector')]//div[@gtdlabelname='" + labelname + "']", this.doc.body, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	var labelsList = this.doc.evaluate( "//div[starts-with(@class,'label-selector')]//span[@gtdlabelname='" + labelname + "'] | //div[starts-with(@class,'label-selector')]//div[@gtdlabelname='" + labelname + "']", this.doc.body, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	var labelEls = [];
	for( var i = 0; i < labelsList.snapshotLength; i++ ) labelEls.push( labelsList.snapshotItem(i) );
	return labelEls;
};

LabelSelectUI.prototype.clickLabel = function(event){
	if( event.button!=0 ) return;
	
	var hadLastEventDetail = this._lastEventDetail;
	this._lastEventDetail = event.detail;
	if( !hadLastEventDetail ) setTimeout( GIPROTOTYPE.bind(this, function(){this.processClickLabel(event)}), 200 );
};
LabelSelectUI.prototype.processClickLabel = function(event){
	// Broadcast to subscribers the label that was clicked
try{
	
	var el = GIPROTOTYPE.element(event);
	
	var lastEventDetail = this._lastEventDetail;
	this._lastEventDetail = null;
	
	var label = {name:el.getAttribute("gtdName"), labelname:el.getAttribute("gtdLabelName"), 'el':el, doubleClick:(lastEventDetail>1)}
	if( !label.labelname ) return;
	this.eventHandlers.broadcast("click-label", label);
	
	//if( this._showHiders["Statuses"] ) this._showHiders["Statuses"]._show(false);
	
	// TODO - RESTORE THIS if( this.settings.ui.hideShowhidersAfterClick ) for( var i = 0; i < this._showHiders.length; i++ ) this._showHiders[i]._show(false);	// hide the show hiders	
	for( t in this._labelSelectors ) this._labelSelectors[t].hideCombos();
	
}catch(e){
	if( GTDDEBUGMODE ) alert("labelselectui clickLabel\n" + e.toString());
}
};

LabelSelectUI.prototype.selected = function(labels){

try{
	
	if( typeof labels=="undefined" ){
		
		// Refresh counts:
		for( t in this._labelSelectors ) this._labelSelectors[t].refreshSelectedCount( this._labelActiveHash );
		
		var clabels = [];
		for( label in this._labelActiveHash ) clabels.push(label);
		return clabels;
	}
	if( typeof labels=="string" ){
		labels = [labels];
	}
	
	
	var cHash = {};
	for( label in this._labelActiveHash ) cHash[label] = true;
	
	var typeCount = {};
	
	for( var i = 0; i < labels.length; i++ ){
		for( var j = 0; j < this._types.length; j++ ){
			if( labels[i].indexOf(this.gtdinboxPage.settings.labels.types[this._types[j]].prefix)==0 ){	
				if( typeCount[this._types[j]] ){
					typeCount[this._types[j]].count++;
					typeCount[this._types[j]].lastLabel = labels[i];
				}else{
					typeCount[this._types[j]] = {count:1, lastLabel: labels[i]};
				}
			}
		}
		
		
		if( cHash[labels[i]] ){	// exists, strip it from known hash
			delete cHash[labels[i]];
		}
		// Create (or refresh existing - because in 2d array)
		var elLabels = this.getLabelsByName(labels[i]);
		if( elLabels ){
			for( var j = 0; j < elLabels.length; j++ ){
				GIPROTOTYPE.addClassName( elLabels[j], "selected" );
			}
			this._labelActiveHash[labels[i]] = true;
		}
	}
	
	// Everything remaining in cHash is to be removed
	for( l in cHash ){
		if( this._labelActiveHash[l] ){
			var elLabels = this.getLabelsByName(l);
			if( elLabels ){
				for( var j = 0; j < elLabels.length; j++ ){
					GIPROTOTYPE.removeClassName( elLabels[j], "selected" );
				}
			}
			delete this._labelActiveHash[l];
		}
	}
	
	// Refresh counts:
	for( t in this._labelSelectors ) this._labelSelectors[t].refreshSelectedCount( this._labelActiveHash );
	
	
	// Update show hiders:
	//this.refreshShowHiderTitles(typeCount);
	
}catch(e){
	if( GTDDEBUGMODE ) alert("labelsui selected\n" + e.toString());
}
};
/*
LabelSelectUI.prototype.refreshShowHiderTitles = function(typeCount){
	for( var j = 0; j < this._types.length; j++ ){
		var t = this._types[j];
		var showHider = this.getShowHiderByType(t);
		if( showHider ){
			if( typeCount && typeCount[t] ){
				if( typeCount[t].count==1 && t=="Statuses" ){
					showHider.setTitle( typeCount[t].lastLabel );
				}else{
					showHider.setTitle( this.gtdinboxPage.stringBundle.getStringSafe("Labels.Types." + t, t) + " (" + typeCount[t].count + ")" );
				}
				GIPROTOTYPE.addClassName(showHider.elTitleTxt, "selected");
			}else{
				showHider.setTitle( this.gtdinboxPage.stringBundle.getStringSafe("Labels.Types." + t, t) );
				GIPROTOTYPE.removeClassName(showHider.elTitleTxt, "selected");
			}
		}
	}
};
*/


LabelSelectUI.prototype.clickCreate = function(event){
try{
	if( this._createWithAjax ){
		//this.gtdinboxPage._moreActionsMenu.createLabelAjax();
		var labelname = prompt(this.gtdinboxPage.stringBundle.getString("Labels.CreateLabel"));	
		if( !labelname ) return;
		
		var finalCallback = GIPROTOTYPE.bind(this, function(){
				this.gtdinboxPage._labels.addNewLabel(labelname);
				this.eventHandlers.broadcast("create-ajax-label", labelname);
			});
		this.gtdinboxPage.gmailDOM.addLabel(labelname, null, finalCallback);
	}else{
		this.gtdinboxPage._moreActionsMenu.createLabel();
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("labelsui clickCreate\n" + e.toString());
}
};

LabelSelectUI.prototype.destroy = function(){
	this.GtdEventObservers.destroy();
	this.eventHandlers.destroy();
	
	this.gtdinboxPage = null;
	this.doc = null;
	this.settings = null;
	this._labels = null;
	
	for( t in this._labelSelectors ){
		this._labelSelectors[t].destroy();
	}
	
	this._elLabelHash = null;
	this._labelActiveHash = null;
	this._elContainers = null;
	this._types = null;
};
