/* Copyright Andy Mitchell 2007, 2008. No warranties */

/*

Two main purposes:
1)
Parse the Gmail Labels box, re-arranging the elements, and extracting all Labels in the account.
Provide functions to retrieve those labels (as new DOM element, array, hash, etc.)

2)
Provide queued interaction with the 'More Actions' drop-down box. 
Used to apply/remove labels to a thread; get current labels on a thread; etc.


*/

function Labels(gtdinboxPage){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this.settings = gtdinboxPage.settings;
	this._eventObservers = new GtdEventObservers(this);
	this._eventHandlers = new GtdEventHandlers_Mutator(this);
	this._showHiders = [];
	this._multiToOne = new MultiToOne(gtdinboxPage, 400);
	this._tmpLabelNames = [];
	this.typeBuckets = {};	// this.typeBuckets[type] = {'labels':{}}
			
	this._labelsBoxCount = new LabelsBoxCount(this._gtdinboxPage);
};

Labels.prototype.refresh = function(event){
	// Called by DOM
	// TODO add a multiToOne here

	if( event ){
		this._multiToOne.process("labelDomChange", GIPROTOTYPE.bind(this, this._refresh));
	}else{
		this._refresh();
	}
};
Labels.prototype._refresh = function(){
	this._tmpLabelNames.length = 0;
	this.load();
};

Labels.prototype.load = function(elNav, tmpLabelNames){
	// Parse the labels box; and re-arrange it at the same time
	
try{
	

	
	// Retrieve the label box
	if( elNav && elNav.nodeType==1 ){
		this.elNav = elNav;
	}
		
	this._gtdinboxPage._perfLogger.countCall("Labels.load");
	
	var typeSettings = this.settings.labels.types;
	
	
	var oldElBox = this.elBox;
	this.elBox = this._gtdinboxPage.gmailDOM.element("labels-box");
	if( !tmpLabelNames && this.elBox.firstChild.className.indexOf("showhider")>-1 ){	// already processed -> stop
		// Just check each entry to see if changed back to prefixed form (& correct it)
		//var result = this._doc.evaluate( "//div[@class='XoqCub XPj4ef']//div[@class='R7iiN c1norb']//div[@class='nb1Q2b']//div[@class='pvSW6e']//div[starts-with(@class,'showhider')]//table//span[@class='zD5BAe']", this.elNav, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
		
		
		
		for( t in this.typeBuckets ){
			for( l in this.typeBuckets[t].labels ){
				var labelObj = this.typeBuckets[t].labels[l];
				if( labelObj.elName.innerHTML.indexOf( typeSettings[t].prefix )==0 ){
					labelObj.elName.innerHTML = labelObj.elName.innerHTML.replace(/.+?(\s(\(|\[)\d+(\)|\]))?$/, labelObj.leafName + "$1");
				}
			}
		}
		
		/*
		var result = this._gtdinboxPage.gmailDOM.elements("labels-box-label-names");
		for( var i = 0; i < result.snapshotLength; i++ ){
			var el = result.snapshotItem(i);
			for( t in this.settings.labels.types ){
				if( el.innerHTML.indexOf(this.settings.labels.types[t].prefix)==0 ){
					el.innerHTML = el.innerHTML.replace(/.+?(\s(\(|\[)\d+(\)|\]))?$/, el.getAttribute("gtdName") + "$1");
				}
			}
		}
		*/
		this._labelsBoxCount.refresh();
		return;
	}
	
	this._lastRefresh = (new Date()).valueOf();
	
		
	
	this._gtdinboxPage._perfLogger.countCall("LabelsBoxCount.load.full");
	
	// Stop recursiveness
	if( this.elObserved ) this._eventObservers.stopObserving( this.elObserved, "DOMNodeInserted", this.refresh, false);
	//if( this.elObserved ) this._eventObservers.stopObserving( this.elObserved, "mousedown", this.mouseDown, true);
	
	
	
	// Destroy old variables:
	var oldLabelsHash = null;
	var labelsChange = null;
	if( tmpLabelNames ){
		labelsChange = {'id':(new Date()).valueOf(), 'changed':true};
	}else{
		oldLabelsHash = this.getLabelsHash(null, true);
		labelsChange = {'id':(new Date()).valueOf(), 'changed':false};
	
		for( var i = 0; i < this._showHiders.length; i++ ){
			this._showHiders[i].destroy();
		}
		this._showHiders.length = 0;
		
		if( this.typeContainerEls ){
			for( t in this.typeContainerEls ){
				delete this.typeContainerEls[t];
			}
			delete this.typeContainerEls;
		}
		
		
		
		if( this.typeBuckets ) delete this.typeBuckets;
		this.typeBuckets = {};
	}

	// Grab the actual original table (contains each label as row)
	this.elTable = this._gtdinboxPage.gmailDOM.element("labels-table");
	
	// Create New Structure for Labels Box:
	if( !tmpLabelNames ){
		this.typeContainerEls = {};
		for( t in typeSettings ){
			this.typeContainerEls[t] = {'elTable':this._doc.createElement("TABLE"), 'elTBody':this._doc.createElement("TBODY")};
				this.typeContainerEls[t].elTable.appendChild( this.typeContainerEls[t].elTBody );
				this.typeContainerEls[t].elTBody.className = "showhider-tbody";
			var sh = new Showhider( this._gtdinboxPage, t, {doc:this._doc, elParent:this.elBox, elBefore:this.elTable, elContent:this.typeContainerEls[t].elTable, title:this._gtdinboxPage.stringBundle.getStringSafe("Labels.Types." + t, t)} );
				sh.elTitle.style.color = "black";
				sh.elTitle.style.fontWeight = "bold";
				sh.show(this.settings.labels.showHideStatus[t]);
				sh.labelType = t;
				sh.addEventHandler( this, "toggle", this._toggleShowHider );
			this._showHiders.push( sh );
		}
	}
	
	
	// Extract labels
	var elRows = this._gtdinboxPage.gmailDOM.elements("labels-box-label-names");
	var labelItems = [];
	if( !tmpLabelNames ){
		var elRows = this._gtdinboxPage.gmailDOM.elements("labels-box-label-names");
		for( var i = 0; i < elRows.snapshotLength; i++ ) labelItems.push( elRows.snapshotItem(i) );
	}
	if( tmpLabelNames ) for( var i = 0; i < tmpLabelNames.length; i++ ) labelItems.push( tmpLabelNames[i] );
	
	for( var i = 0; i < labelItems.length; i++ ){
		
		var labelname = "";
		var elRow = null;
		var elName = null;
		
		if( typeof labelItems[i]=="string" ){
			labelname = labelItems[i];
		}else{
			var elName = elRows.snapshotItem(i);
			var elRow = elName.parentNode;
			while( elRow.parentNode && elRow.nodeName!="TR" ){
				elRow = elRow.parentNode;
			}
			var labelname = elName.getAttribute("gtdLabelName");
			if( !labelname ) labelname = elName.innerHTML.replace(/\s\(\d+\)$/, "");		
			elName.setAttribute( "gtdLabelName", labelname );
		}
		
		
		var foundType = "";
		var labelSansPrefix = labelname;
		for( t in typeSettings ){
			if( labelname.indexOf( typeSettings[t].prefix )==0 ){
				foundType = t;
				labelSansPrefix = labelSansPrefix.replace(typeSettings[t].prefix,"");
				break;
			}			
		}
		if( !foundType ) foundType = "Misc";
		
		if( !this.typeBuckets[foundType] ) this.typeBuckets[foundType] = {'labels':{}};
		
		var subs = labelSansPrefix.split("/");
		var compositeLabelname = typeSettings[foundType].prefix || "";
		var depth = 0;
		var parentLabelname = "";
		while( subs.length > 0 ){
		
			var currentLabelname = subs.shift();
			if( depth > 0 ) compositeLabelname += "/";
			compositeLabelname += currentLabelname;
			
			// Build data object
			
			
			var parent = (depth>0? this.typeBuckets[foundType].labels[parentLabelname] : null);
			if( !this.typeBuckets[foundType].labels[compositeLabelname] ){	// Create new entry for it
				this.typeBuckets[foundType].labels[compositeLabelname] = new LabelData(this._gtdinboxPage);
				this.typeBuckets[foundType].labels[compositeLabelname].init( foundType, currentLabelname, compositeLabelname, depth, parent );
			}
			var labelObj = this.typeBuckets[foundType].labels[compositeLabelname];
			
			if( tmpLabelNames ){
				
				// Transform elements
				if( !labelObj.el ){
					// Create place holder
					labelObj.el = this._doc.createElement("TR");
						labelObj.el.className = "placeholder-label";
					var elTD = this._doc.createElement("TD");
					var elDivOuter = this._doc.createElement("DIV");
						elDivOuter.setAttribute("style", "overflow: hidden; white-space: nowrap;");
					var elDiv = this._doc.createElement("DIV");
						elDiv.className = "yyT6sf";
					labelObj.elName = this._doc.createElement("SPAN");
						labelObj.elName.innerHTML = currentLabelname;
					elDiv.appendChild( labelObj.elName );
					elDivOuter.appendChild( elDiv );
					elTD.appendChild( elDivOuter );
					labelObj.el.appendChild( elTD );
					var elTD2 = this._doc.createElement("TD");
						elTD2.className = "BFvfre";
					labelObj.el.appendChild( elTD2 );
					
					if( subs.length==0 ){	// It's a temporary, but ultimately real, label
						labelObj.isPlaceHolder = false;
						
						var tmpLabelsHash = this.getLabelsHash();
						for( l in tmpLabelsHash ){
							if( !tmpLabelsHash[l].isPlaceHolder ){
								labelObj.el.className = tmpLabelsHash[l].el.className;
								break;
							}
						}
						
					}
					
					if( labelObj.el.getAttribute("gtdLabelFullName")!=compositeLabelname ){
						labelObj.el.setAttribute("gtdType", foundType);
						labelObj.el.setAttribute("gtdName", currentLabelname);
						if( !labelObj.isPlaceHolder ) labelObj.el.setAttribute("gtdLabelName", compositeLabelname);
						labelObj.el.setAttribute("gtdLabelFullName", compositeLabelname);
					}

					
					if( depth > 0 ){ // Ensure I'm indented
						labelObj.transformToIndent();
					}
					labelObj.elName.parentNode.style.marginLeft = "12px";
					
					if( !labelObj.movedIntoNewStructure ){
						if( parent ){
							parent.transformToIncludeExpander();
							labelObj.el.setAttribute("gtdLabelParentName", parent.labelName);
							var firstChild = parent.getFirstChild();	// get sibling
							if( firstChild==labelObj ){	// it's me! Behave as normal
								this.typeContainerEls[foundType].elTBody.appendChild( labelObj.el );
								labelObj.el.style.display = "none";
							}else{
								firstChild.el.parentNode.insertBefore( labelObj.el, firstChild.el );
								labelObj.el.style.display = firstChild.el.style.display;	// Mimic open/close of sibling
							}
						}else{
							this.typeContainerEls[foundType].elTBody.appendChild( labelObj.el );
						}
						labelObj.movedIntoNewStructure = true;
					}
				}
				
			}else{
				if( elName && subs.length==0 ){	// final one - real label for an element
					elName.innerHTML = elName.innerHTML.replace(/.+?(\s\(\d+\))?$/, currentLabelname + "$1");
					labelObj.el = elRow;
					labelObj.elName = elName;
					labelObj.isPlaceHolder = false;
					
					if( oldLabelsHash ){
						if( labelObj._changeCheck!=labelsChange.id ){
							labelObj._changeCheck = labelsChange.id;
							
							if( oldLabelsHash[compositeLabelname] ){
								delete oldLabelsHash[compositeLabelname];
							}else{	// Added something
								labelsChange.changed = true;
							}
						}
					}
				}
				
				// Transform elements
				if( labelObj.isPlaceHolder && !labelObj.el ){
					// Create place holder
					labelObj.el = this._doc.createElement("TR");
						labelObj.el.className = "placeholder-label";
					var elTD = this._doc.createElement("TD");
					var elDivOuter = this._doc.createElement("DIV");
						elDivOuter.setAttribute("style", "overflow: hidden; white-space: nowrap;");
					var elDiv = this._doc.createElement("DIV");
						elDiv.className = "yyT6sf";
					labelObj.elName = this._doc.createElement("SPAN");
						labelObj.elName.innerHTML = currentLabelname;
					elDiv.appendChild( labelObj.elName );
					elDivOuter.appendChild( elDiv );
					elTD.appendChild( elDivOuter );
					labelObj.el.appendChild( elTD );
					var elTD2 = this._doc.createElement("TD");
						elTD2.className = "BFvfre";
					labelObj.el.appendChild( elTD2 );
				}
				if( subs.length==0 && !elName && tmpLabelNames ){	// It's a temporary, but ultimately real, label
					labelObj.isPlaceHolder = false;
					
					var tmpLabelsHash = this.getLabelsHash();
					for( l in tmpLabelsHash ){
						if( !tmpLabelsHash[l].isPlaceHolder ){
							labelObj.el.className = tmpLabelsHash[l].el.className;
							break;
						}
					}
					
				}
				
				if( labelObj.el.getAttribute("gtdLabelFullName")!=compositeLabelname ){
					labelObj.el.setAttribute("gtdType", foundType);
					labelObj.el.setAttribute("gtdName", currentLabelname);
					if( !labelObj.isPlaceHolder ) labelObj.el.setAttribute("gtdLabelName", compositeLabelname);
					labelObj.el.setAttribute("gtdLabelFullName", compositeLabelname);
				}

				if( parent ){
					// If parent has el; and is not transformed; make sure it is transformed. And as a child, make sure I'm hidden.
					parent.transformToIncludeExpander();
					labelObj.el.style.display = "none";
					labelObj.el.setAttribute("gtdLabelParentName", parent.labelName);
				}
				if( depth > 0 ){
					// Ensure I'm indented
					labelObj.transformToIndent();
				}
				labelObj.elName.parentNode.style.marginLeft = "12px";
				
				// Add Popout Icon
				if( GTDDEVMODE ){
					if( !labelObj.addedPopoutIcon && !labelObj.isPlaceHolder ){
						var elPopout = this._doc.createElement("TD");
							elPopout.innerHTML = "<img src='chrome://gtdinbox/content/filter-extend.png' class='gtd-popout-icon'/>";
							elPopout.firstChild.title = this._gtdinboxPage.stringBundle.getString("Labels.Popout.Title");
						labelObj.el.appendChild( elPopout );
						labelObj.addedPopoutIcon = true;
					}
				}
				
				// Move the element to new structure
				if( !labelObj.movedIntoNewStructure ){
					this.typeContainerEls[foundType].elTBody.appendChild( labelObj.el );
					labelObj.movedIntoNewStructure = true;
				}
			}
						
			// Prep for next label
			parentLabelname = compositeLabelname;
			depth++;
		}
		
		
	}
	
	// Rearrange Statuses
	if( this.typeBuckets.Statuses ){
		var defaultStatuses = [];
		for( d in typeSettings.Statuses.defaults ){
			defaultStatuses.push( typeSettings.Statuses.prefix + typeSettings.Statuses.defaults[d] )
		}
		defaultStatuses.reverse();
		for( var i = 0; i < defaultStatuses.length; i++ ){
			if( this.typeBuckets.Statuses.labels[defaultStatuses[i]] ){
				this.typeContainerEls.Statuses.elTBody.insertBefore( this.typeBuckets.Statuses.labels[defaultStatuses[i]].el, this.typeContainerEls.Statuses.elTBody.firstChild );
			}
		}		
	}
	
	
	// See if need to hide any empty boxes
	for( var i = 0; i < this._showHiders.length; i++ ){
		var sh = this._showHiders[i];
		if( this.typeContainerEls[sh.labelType].elTBody.innerHTML.length==0 ){	// Nothing doing
			sh.visible(false);
		}else{
			sh.visible(true);
		}
	}

	
	if( oldElBox ){
		this._eventObservers.stopObserving( oldElBox, "mouseup", this.labelsSectionMousedown, true);
		this._eventObservers.stopObserving( oldElBox, "mousedown", this.testStopEvent, true);
		this._eventObservers.stopObserving( oldElBox, "click", this.testStopEvent, true);
	}
	this._eventObservers.observe( this.elBox, "mouseup", this.labelsSectionMousedown, true);
	this._eventObservers.observe( this.elBox, "mousedown", this.testStopEvent, true);
	this._eventObservers.observe( this.elBox, "click", this.testStopEvent, true);
	
	
	
	if( !tmpLabelNames ) this._labelsBoxCount.refresh(true);
	
	this.elObserved = this.elBox.parentNode.parentNode.parentNode;
	this._eventObservers.observe( this.elObserved, "DOMNodeInserted", this.refresh, false);
	//this._eventObservers.observe( this.elObserved, "mousedown", this.mouseDown, true);
	

	
	if( oldLabelsHash ){
		if( GIPROTOTYPE.hashLength(oldLabelsHash) > 0 ){	// Something deleted or renamed
			labelsChange.changed = true;
		}
	}
	
	this._eventHandlers.broadcast("refresh-labels", labelsChange);
	
}catch(e){
	if( GTDDEBUGMODE ) alert("labels load\n" + e.toString());
}
try{	
	
}catch(e){
	if( GTDDEBUGMODE ) alert("labels load 2\n" + e.toString());
}
};

Labels.prototype._toggleShowHider = function(sh){
	// Called when ShowHider toggled
	var isShowing = sh.show();
	if( this.settings.labels.showHideStatus[sh.labelType]!=isShowing ){
		this.settings.labels.showHideStatus[sh.labelType] = isShowing;
		this._gtdinboxPage.saveSettings();
	}
};


/************************************************************************************
 ** labelsSectionMousedown - event handler for mousedown of a label section        **
 **   return value: none                                                           **
 **                                                                                **
 ** event          event  event object passed to the function
 ************************************************************************************/
Labels.prototype.labelsSectionMousedown = function(event, typeBucket){
try{
	
	var elClicked = GIPROTOTYPE.element(event);
    
	if( elClicked.className == "label-folder-toggle" ){
		var elInspected = elClicked;
		while( elInspected.parentNode ){
			elInspected = elInspected.parentNode;
			if(( elInspected.className == 'WnjAod') || GIPROTOTYPE.hasClassName(elInspected, 'placeholder-label') || GIPROTOTYPE.hasClassName(elInspected, "label") ){
				
				if( !typeBucket ){
					typeBucket = this.typeBuckets[ elInspected.getAttribute("gtdType") ];
				}
				var label = typeBucket.labels[ elInspected.getAttribute("gtdLabelFullName") ];
				
				if( elClicked.src.indexOf("closed.png")>-1 ){	// It's closed, so open it
					label.openLabel();
				}else{	// it's open, close it
					label.closeLabel();
				}
				break;
			}
		}
	}else if( elClicked.className=="gtd-popout-icon" ){
		var elInspected = elClicked;
		while( elInspected.parentNode ){
			elInspected = elInspected.parentNode;
			if(( elInspected.className == 'WnjAod') || GIPROTOTYPE.hasClassName(elInspected, 'placeholder-label') || GIPROTOTYPE.hasClassName(elInspected, "label") ){
				var type = elInspected.getAttribute("gtdType");
				var labelName = elInspected.getAttribute("gtdLabelFullName");
				this._gtdinboxPage._filterPopupUI.refresh(labelName, type, true);
				this._gtdinboxPage._filterPopupUI.popup(elClicked,{location:'right', 'margin':5});
				break;
			}
		}
	}else{
		for( l in this.typeBuckets.Statuses.labels ){
			var label = this.typeBuckets.Statuses.labels[l];
			if( elClicked==label.elName ){
				var status = this._gtdinboxPage.getStatusData();
				if( label.labelName!=status.Finished ){
					GIPROTOTYPE.stop(event);
					if( label.labelName==status.Action ){
						this._gtdinboxPage._searchBoxUI.search("(" + GIPROTOTYPE.searchableLabel(label.labelName) + " OR " + GIPROTOTYPE.searchableLabel( status.NextAction) + ") -" + GIPROTOTYPE.searchableLabel( status.Finished));
					}else{
						this._gtdinboxPage._searchBoxUI.search(GIPROTOTYPE.searchableLabel(label.labelName) + " -" + GIPROTOTYPE.searchableLabel( status.Finished));
					}
				}
			}
		}
	}
}catch(e){
	if( GTDDEBUGMODE ) alert("Labels labelsSectionMousedown\n" + e.toString());
}
};

Labels.prototype.testStopEvent = function(event){
	var elClicked = GIPROTOTYPE.element(event);
	for( l in this.typeBuckets.Statuses.labels ){
		if( elClicked==this.typeBuckets.Statuses.labels[l].elName ){
			var status = this._gtdinboxPage.getStatusData();
			if( this.typeBuckets.Statuses.labels[l].labelName!=status.Finished ){
				GIPROTOTYPE.stop(event);
			}
		}
	}
};


	



Labels.prototype.addNewLabel = function(labelname){
	// Hack-job. Used if new label created; but Gmail doesn't update fast enough. Called by thread.addLabel.

	this._tmpLabelNames.push(labelname);
	this.load(null, this._tmpLabelNames);
	
};

Labels.prototype.getLabelsEl = function(type){

try{
	
	var typeObject = {'labels':{}, 'el':null};
	if( !this.typeBuckets[type] ) return typeObject;

	var el = this._doc.createElement("DIV");
		el.className = "labels-group";
		el.innerHTML = "<div class='title'>" + this._gtdinboxPage.stringBundle.getStringSafe("Labels.Types." + type, type) + "</div>";
	var elLabels = this._doc.createElement("DIV");
		elLabels.className = "labels";
	el.appendChild( elLabels );
	
	for( l in this.typeBuckets[type].labels ){
		if( !this.typeBuckets[type].labels[l].parent ){	// Only clone roots. They clone below.
			this.typeBuckets[type].labels[l].clone(elLabels, typeObject.labels);
		}
	}
	
	typeObject.el = el;
	//this._eventObservers.observe( elLabels, "mousedown", function(event){this.labelsSectionMousedown(event, typeObject)}, false);
	elLabels.addEventListener( "mousedown", GIPROTOTYPE.bind( this, function(event){this.labelsSectionMousedown(event, typeObject)}), false);
	
	return typeObject;
}catch(e){
	if( GTDDEBUGMODE ) alert( "Labels getLabelsEl\n" + e.toString());
}
};


Labels.prototype.getLabelsArray = function(types, sortStatuses, removePlaceHolders){

try{
	if( !types ) types = ["_all"];
	else if( typeof types=="string" ){
		types = [types];
	}
	
	
	var labelsArray = [];
			
	if( types.length==1 ){
		if( types[0]=="_all" ){
			for( t in this.typeBuckets ){
				for( l in this.typeBuckets[t].labels ) if( !(removePlaceHolders && this.typeBuckets[t].labels[l].isPlaceHolder) ) labelsArray.push(l);
			}
		}else{
			if( sortStatuses && types[0]=="Statuses" ){
				var statuses = this.getLabelsArray("Statuses");
				var defaults = this._gtdinboxPage.settings.labels.types.Statuses.defaults;
				var status = this._gtdinboxPage.getStatusData();
				for( d in defaults ){
					for( var i = statuses.length - 1; i >=0; i-- ){
						if( statuses[i]==status[d] ){
							labelsArray.push(statuses[i]);
							statuses.splice(i, 1);
							break;
						}
					}
				}
				for( var i = 0; i < statuses.length; i++ ) labelsArray.push(statuses[i]);
			}else{
				for( l in this.typeBuckets[types[0]].labels ) if( !(removePlaceHolders && this.typeBuckets[types[0]].labels[l].isPlaceHolder) ) labelsArray.push(l);
			}
		}
	}else{
		for( var i = 0; i < types.length; i++ ){
			for( l in this.typeBuckets[types[i]].labels ) if( !(removePlaceHolders && this.typeBuckets[types[i]].labels[l].isPlaceHolder) ) labelsArray.push(l);
		}
	}
	
	
	return labelsArray;

}catch(e){
	if( GTDDEBUGMODE ) alert( "Labels getLabelsArray\n" + e.toString());
}
};

Labels.prototype.getLabelsHash = function(types, removePlaceHolders){
	if( !types ) types = ["_all"];
	else if( typeof types=="string" ) types = [types];
	
	var labelsHash = {};
			
	if( types.length==1 ){
		if( types[0]=="_all" ){
			for( t in this.typeBuckets ){
				for( l in this.typeBuckets[t].labels ){
					 if( !(removePlaceHolders && this.typeBuckets[t].labels[l].isPlaceHolder) ) labelsHash[l] = this.typeBuckets[t].labels[l];
				}
			}
		}else{
			for( l in this.typeBuckets[types[0]].labels ){
				 if( !(removePlaceHolders && this.typeBuckets[types[0]].labels[l].isPlaceHolder) ) labelsHash[l] = this.typeBuckets[types[0]].labels[l];
			}
		}
	}else{
		for( var i = 0; i < types.length; i++ ){
			for( l in this.typeBuckets[types[i]].labels ){
				 if( !(removePlaceHolders && this.typeBuckets[types[i]].labels[l].isPlaceHolder) ) labelsHash[l] = this.typeBuckets[types[i]].labels[l];
			}
		}
	}
	return labelsHash;
};

Labels.prototype.hasLabel = function(labelname, removePlaceHolders){
	// TODO This is used quite a lot; could cache the labels found into a hash for faster recall
	for( t in this.typeBuckets ){
		if( this.typeBuckets[t].labels[labelname] && !(removePlaceHolders && this.typeBuckets[t].labels[labelname].isPlaceHolder) ){
			return true;
		}
	}
	return false;
};

Labels.prototype.getLabelsQuery = function(type, restrictArr, removeArr, exclude, overrideLabels){
	var search = "";
	var labels = overrideLabels || this.getLabelsArray(type, true);
	if( (typeof labels=="object") && !(labels instanceof Array) ) labels = GIPROTOTYPE.hashKeysToArray(labels);
	var restrictHash = restrictArr? GIPROTOTYPE.arrayToHash(restrictArr) : null;
	var removeHash = removeArr? GIPROTOTYPE.arrayToHash(removeArr) : null;
		
	for( var i = (labels.length - 1); i>=0; i-- ){
		if( !(restrictHash || removeHash) || (restrictHash && restrictHash[labels[i]]) || (removeHash && !removeHash[labels[i]]) ){
			search += GIPROTOTYPE.searchableLabel( labels[i] ) + " OR ";
		}
	}
	search = search.replace(/ OR $/,"");
	if( search ) search = "(" + search + ")";
	if( exclude ){
		if( typeof exclude=="string" ){
			search = "(" + search + " -" + GIPROTOTYPE.searchableLabel(exclude) + ")";
		}else if( typeof exclude=="array"){
			for( var i = 0; i < exclude.length; i++ ){
				search += " -" + GIPROTOTYPE.searchableLabel(exclude[i]);
			}
			search = "(" + search + ")";
		}
	}
	return search;
};


Labels.prototype.createBasicLabels = function(callback, fCallback){
try{
	// Using the current prefix (set correctly elsewhere); see if any labels are missing. If so, add them.
		
	var at = this._gtdinboxPage.gmailDOM.local("at");
	if( !callback && !at ){
		setTimeout( GIPROTOTYPE.bind(this, function(){this.createBasicLabels(1, fCallback)}), 3000);	// need correct environment variables to load before can ajax call gmail
		return;
	}else if( callback==1 && !at ){	// Try again, having recreated the basic at
		//this._detectAt(null, GIPROTOTYPE.bind(this, function(){this.createBasicLabels(2, fCallback)}), true );
		//return;
	}else if( !at ){
		if( fCallback ) fCallback(false);
		return;	// Cannot proceed without at variable
	}
	
	
	
	
	var settings = this._gtdinboxPage.settings;
	//settings.labels.types["Statuses"]
	
	var labels = [];
	for( t in settings.labels.types ){
		if( settings.labels.types[t].defaults ){
			var prefix = settings.labels.types[t].prefix;
			for( d in settings.labels.types[t].defaults ){
				var label = prefix + settings.labels.types[t].defaults[d];
				if( !this._labels.hasLabel(label) ) labels.push(label);
			}
		}
	}
	
	
	
	var finalCallback = GIPROTOTYPE.bind(this, function(){
				try{
					this._gtdinboxPage.gmailDOM.refreshView();
				}catch(e){
					if( GTDDEBUGMODE ) alert("lightbox createbasiclabels final callback\n" + e.toString());
					if( fCallback ) fCallback(true);
				}
				if( fCallback ) fCallback(true);
				alert( this._gtdinboxPage.getStringBundle("Labels.CreateBasicLabels.Complete") );
			});
	if( labels.length>0 ){
		if( !confirm( this._gtdinboxPage.stringBundle.getString("ConfirmInstall") ) ){
			alert(this._gtdinboxPage.stringBundle.getString("ConfirmInstall.No"));
			return;
		}
		var at = this._gtdinboxPage.gmailDOM.addLabel(labels, null, finalCallback);
	}else{
		if( fCallback ) fCallback(true);
	}
	
	

}catch(e){
	if( GTDDEBUGMODE ) alert("labels createbasiclabels\n" + e.toString());
	if( fCallback ) fCallback(true);
}

};





Labels.prototype.destroy = function(){
	for( var i = 0; i < this._showHiders.length; i++ ){
		this._showHiders[i].destroy();
	}
	this._multiToOne.destroy();
	this._showHiders.length = 0;
	this._eventObservers.destroy();
};