/* Copyright Andy Mitchell 2007, 2008. No warranties */

function GtdLightbox(gtdinboxPage, elParent, type){
	this._gtdinboxPage = gtdinboxPage;
	this._doc = gtdinboxPage.doc;
	this._labels = gtdinboxPage._labels;
	this._eventObservers = new GtdEventObservers(this);
	this._eventHandlers = new GtdEventHandlers_Mutator(this);
	//this._xmlHttpRequest = new GIXMLHttpRequest(gtdinboxPage);
	
	this._create(elParent);
	this.setType(type);
};


GtdLightbox.prototype._create = function(elParent){
	this.el = this._doc.createElement("DIV");
		this.el.className = "lightbox";
		this.el.setAttribute("style", "display:none;");
	if( elParent ) elParent.appendChild( this.el );
	
	this.elContainer = this._doc.createElement("DIV");
		this.elContainer.className = "lightbox-container";
		this.elContainer.setAttribute("style", "margin:auto;z-index:5001;width:800px;");
	this.el.appendChild( this.elContainer );
};

GtdLightbox.prototype.setType = function(type, options){
try{
	options = options || {};
	this._type = type;
	this._instance = (new Date()).valueOf();
	

	
	var el = this._doc.createElement("DIV");
	if( type && type.nodeName && (typeof type!="string") ){
		this.content( type );
	}else if( type=="welcome" || type=="update" ){
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile(type) );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		this.content(el);
				
		var elMain = this._doc.getElementById("gtd-main-info-box");
		if( type=="welcome" ){
			var el = this._doc.getElementById("link-overview");
			this._eventObservers.observe(el, "mousedown", function(event){
					GIPROTOTYPE.stop(event);
					elMain.scrollTop = 0;
				}, false);
			
			var el = this._doc.getElementById("link-setuplabels");
			this._eventObservers.observe(el, "mousedown", function(event){
					var el = this._doc.getElementById("header-setuplabels");
					GIPROTOTYPE.stop(event);
					el.scrollIntoView();
				}, false);
		}
		
		var elNews = this._doc.getElementById("recent-news");
		var newsBox = new NewsBox(this._gtdinboxPage);
		elNews.appendChild( newsBox.el );
		newsBox.refresh();
		
		var el = this._doc.getElementById("labels-adddefault");
		var status = this._gtdinboxPage.getStatusData();
		var needsBasic = false;
		for( d in this._gtdinboxPage.settings.labels.types.Statuses.defaults ){
			if( !this._gtdinboxPage._labels.hasLabel(status[d]) ){
				needsBasic = true;
				break;
			}
		}
		el.disabled = needsBasic? false : true;
		this._eventObservers.observe(el, "mousedown", function(event){
				el.disabled = true;
				this._gtdinboxPage._labels.createBasicLabels(null, GIPROTOTYPE.bind(this, function(){
				}));
			}, false);
		var el = this._doc.getElementById("labels-golabelscreen");
		this._eventObservers.observe(el, "mousedown", function(event){
				var el = this._gtdinboxPage.gmailDOM.element("edit-labels");
				if( el ) GIPROTOTYPE.doMouseEvent(this._gtdinboxPage.gmailDOM.element("document"), el, "click");
				this.active(false);
			}, false);
				
		
	}else if( type=="store" ){
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("store") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		
		this.content(el);
		
		var doc = this.elContent.ownerDocument;
		var elClose = doc.getElementById("gtdinbox.store.close");
		this._eventObservers.observe(elClose, "mousedown", GIPROTOTYPE.bind(this, function(event){
				this.active(false);
			}), false);
	}else if( type=="printer" ){
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("printer") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		
		this.content(el);
		
		var doc = this.elContent.ownerDocument;
		var elClose = doc.getElementById("gtd.cancel");
		this._eventObservers.observe(elClose, "mousedown", GIPROTOTYPE.bind(this, function(event){
				this.active(false);
			}), false);
		
	}else if( type=="autolabelling" ){
	
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("autolabelling") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		this.content(el);
		
		this._setCountdown(40000);	// make it turn off in 30 seconds no matter what
	}else if( type=="processing" ){
	
		var el = this._doc.createElement("DIV");
		el.style.margin = "auto";
		el.style.textAlign = "center";
		//el.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif'/>";
		this.content(el);
		if( options.msg ) el.innerHTML = "<div class='gtdwaiting'>" + options.msg + "</div>"
		
		this._setCountdown(10000);	// make it turn off in 30 seconds no matter what
	}else if( type=="searching" ){
	
		var el = this._doc.createElement("DIV");
		el.style.margin = "auto";
		el.style.textAlign = "center";
		el.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_medium.gif'/>";
		this.content(el);
		if( options.msg ) el.innerHTML += "<div class='gtdwaiting'>" + options.msg + "</div>"
		
		this._setCountdown(180000);	// make it turn off in 180 seconds no matter what
	}else if( type=="disclaimer" ){
	
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("disclaimer") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		this.content(el);
		
		
		var doc = this.elContent.ownerDocument;
		var elClose = doc.getElementById("gtdinbox.disclaimer.close");
		this._eventObservers.observe(elClose, "mousedown", GIPROTOTYPE.bind(this, function(event){
				this.active(false);
			}), false);
				
		
	}else if( type=="resolve-limited" ){
	
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("resolve-limited") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		this.content(el);
		
		
		var doc = this.elContent.ownerDocument;
		
		if( !this._gtdinboxPage.gmailDOM.local("at") ){
			doc.getElementById("gtdinbox.resolve-limited.at").style.display = "";
		}
		if( !this._gtdinboxPage._threadList.hasFullBodiedThreads() ){
			doc.getElementById("gtdinbox.resolve-limited.nothreads").style.display = "";
		}
					
		doc.getElementById("gtdinbox.resolve-limited.fixgmail").style.display = "";
		doc.getElementById("gtdinbox.resolve-limited.close").style.display = "";
		doc.getElementById("gtdinbox.resolve-limited.loading").style.display = "none";
		
		
		var elFix = doc.getElementById("gtdinbox.resolve-limited.fixgmail");
		this._eventObservers.observe(elFix, "mousedown", GIPROTOTYPE.bind(this, function(event){
				doc.getElementById("gtdinbox.resolve-limited.fixgmail").style.display = "none";
				doc.getElementById("gtdinbox.resolve-limited.close").style.display = "none";
				doc.getElementById("gtdinbox.resolve-limited.loading").style.display = "";
				//this._detectAt(null, GIPROTOTYPE.bind(this, function(){this.active(false)}) );
			}), false);
		
		
		var elClose = doc.getElementById("gtdinbox.resolve-limited.close");
		this._eventObservers.observe(elClose, "mousedown", GIPROTOTYPE.bind(this, function(event){
				this.active(false);
			}), false);
				
		
	}else if( type=="updatingmessages" ){
	
		var msg = GTDFileIO.readFromUrl( this._gtdinboxPage.gtdinboxBrowser.getLocalHtmFile("updatingmessages") );
		var match = msg.match(/<body[^>]*>((.|\r|\n|\t|\e|\f|\a|\v)+)<\/body>/i);
		el.innerHTML = match[1];
		this.content(el);
		
		this._setCountdown(40000);	// make it turn off in 30 seconds no matter what
		
	}else if( type=="debug" ){
		var html = [];
		html.push("<div class='gtdmsg'>");
		html.push("<button id='gtdinbox.debug.close'>Close</button>");
		html.push("GTDInbox: Different Gmails Debugger");
		html.push("View: <span id='gtdinbox.debug.view.tl'>Thread List</span>, <span id='gtdinbox.debug.view.cv'>Conversation</span>, <span id='gtdinbox.debug.view.co'>Compose</span>, <span id='gtdinbox.debug.view.ct'>Contacts</span>");
		html.push("");
		html.push("There are many oh-so-slightly different variants of Gmail running in the world, which can de-stabalise GTDInbox (as it's developed on just my machine, and therefore just my Gmail)");
		html.push("The reason being I suspect Gmail does constant rolling global deployment, one server at a time, meaning users on different servers see slightly different things");
		html.push("(For the record, it seems language plays a role... i.e. English US get updates before almost anyone else)");
		html.push("");
		html.push("What symptoms will you get? Well, GTDInbox may appear to glitch, in a way that can be attributed to it failing to find part of the page.");
		html.push("");
		html.push("What is this? As I cannot actually play with your version of Gmail to diagnose the problem, I have to hope enough tech-savvy users experience the differences and are able to tell me what's failing and how to fix it.");
		html.push("To help with this, this Debug window trys to find all the common GTDInbox-related elements, using a custom XPath directive, and tells you when it fails (or more subtly, when the inner HTML - aka text - is not as expected).");
		html.push("If you want to dig in, load up the source (search for GmailDOM.js on your machine - it has code for all the element keys), and load at the XPath statement. This is what will most likely fail. If you use Firebug - and enable it in Gmail - you can then follow the failed XPath and try to work out your own XPath that will work on your Gmail. If you then send that to me, I can incorporate it into the general release; the fixing the issue for everyone");
		html.push("");
		html.push("<table><tbody id='gtdinbox.debug'><tr><td width='30%'>Element Key in GmailDOM.js</td><td width='10%'>Found</td><td width='60%'>Found Inner HTML</td></tr></tbody></table>");
		html.push("</div>");
		
		el.innerHTML = html.join("<br/>");
		this.content(el);
		
		var doc = this.elContent.ownerDocument;
		
		var view = this._gtdinboxPage.view();
		var elView = doc.getElementById("gtdinbox.debug.view." + view);
		if( elView ) elView.style.fontWeight = "bold";
		
		var elClose = doc.getElementById("gtdinbox.debug.close");
		this._eventObservers.observe(elClose, "mousedown", GIPROTOTYPE.bind(this, function(event){
				this.active(false);
			}), false);
				
		var elTBody = doc.getElementById("gtdinbox.debug");
		var renderElement = GIPROTOTYPE.bind(this, function(key, notes){
			var el = this._gtdinboxPage.gmailDOM.element(key);
			var elTr = this._doc.createElement("TR");
				elTr.setAttribute("valign","top");
			var elTd = this._doc.createElement("TD");
				elTd.innerHTML = key;
				elTd.setAttribute("valign","top");
			elTr.appendChild(elTd);
			var elTd = this._doc.createElement("TD");
				elTd.innerHTML = !!(el);
			elTr.appendChild(elTd);
			var elTd = this._doc.createElement("TD");
				if( el ) elTd.innerHTML = GIPROTOTYPE.decodeEntities(el.innerHTML.replace(/<[^>]+>/g, ""));
				if( el && el.value ) elTd.innerHTML += "<br/>(Value: " + el.value + ")";
				if( notes ) elTd.innerHTML += "<br/><i>[" + notes + "]</i>";
			elTr.appendChild(elTd);
			
			elTBody.appendChild(elTr);
		});
		
		renderElement("action-combo");
		renderElement("action-combo-list", "May be false. That's ok.");
		renderElement("email-address");
		renderElement("alert-box", "No inner HTML");
		renderElement("search-box", "No inner HTML");
		renderElement("search-button");
		renderElement("search-row");
		renderElement("nav-buckets");
		renderElement("inbox-link");
		renderElement("compose-mail");
		
		if( view=="tl" ){
			renderElement("archive-button");
			renderElement("spam-button");
			renderElement("delete-button");
			renderElement("threadlist-refresh");
		}
		
		if( view=="cv" ){
			renderElement("conversation-header");
			renderElement("conversation-subject");
		}
		
		if( view=="co" ){
			renderElement("plain-textarea", "No inner HTML");
			renderElement("compose-to", "No inner HTML");
			renderElement("compose-table" );
		}
		
		if( view=="ct" ){
			renderElement("contacts-frame-body");
		}
		
	};
	
	
	var elCancel = this._doc.getElementById("cancel");
	if( elCancel ){
		this._eventObservers.observe(elCancel, "mousedown", GIPROTOTYPE.bind(this, function(event){
			this.active(false);
		}), false);
	}

}catch(e){
	if( GTDDEBUGMODE ) alert("GtdLightbox setType\n" + e.toString());
	this.active(false);
}
};



GtdLightbox.prototype._setCountdown = function(time){
	var t = this._type;
	var i = this._instance;
	setTimeout( GIPROTOTYPE.bind(this, function(){
			if( this.active() && t==this._type && i==this._instance ){	// still on same type
				this.active(false);
			}
		}), time);
};

/*
GtdLightbox.prototype._retainOldPrefixes = function(){
	// For each prefix, if any instance (label) with old prefix; keep that old prefix. If not, use new prefix.
	
	var labels = this._labels.getLabelsArray();
	var settings = this._gtdinboxPage.settings;
	var options = this._gtdinboxPage.gtdinboxBrowser.options;
	for( l in settings.labels.types ){
		var oldKey = l.replace(/s$/, "");
		if( /se$/.test(oldKey) ) oldKey = oldKey.replace(/e$/, "");
		var oldPrefix = options.getOption("char", "GTDInbox_Prefix_" + oldKey );
		var found = false;
		for( var i = 0; i < labels.length; i++ ){
			if( labels[i].indexOf(oldPrefix)==0 ){	// Old Prefix matches, retain it
				found = true;
				break;
			}
		}
		if( found ){
			this._gtdinboxPage.settings.labels.types[l].prefix = oldPrefix;
		}else{
		}
	}
	this._gtdinboxPage.saveSettings();
};
*/





GtdLightbox.prototype.active = function(v){
	if( typeof v=="undefined" ) return this.el.style.display!="none";
	if( v ){
		this._eventHandlers.broadcast("beforeactivate");
		this.el.style.display = "";
		this._eventObservers.observe(window, "resize", this._position, false );
		this._eventHandlers.broadcast("activated");
		this._position();
		
	}else{
		this.el.style.display = "none";
		this._eventObservers.stopObserving(window, "resize", this._position, false );
		this._eventHandlers.broadcast("deactivate");
		this.lastDims = null;
	}
		
};

GtdLightbox.prototype.content = function(el){
try{
	if( typeof el=="undefined" ) return this.elContent;
	
	while( this.elContainer.firstChild ) this.elContainer.removeChild( this.elContainer.firstChild );
	
	
	this.elContent = el;
	this.elContainer.appendChild( el );
	
	this._position();
}catch(e){
	this.active(false);
	if( GTDDEBUGMODE ) alert("GtdLightbox content\n" + e.toString());
}
};
GtdLightbox.prototype._position = function(){
	// Correctly position container, based on its content's dimensions

try{
	var winDims = GIPROTOTYPE.viewport(this._doc, this._gtdinboxPage.window);
	var cDims = GIPROTOTYPE.getDimensions(this.elContainer, this._doc);
				
	this.el.style.height = winDims[1] + "px";
	this.el.style.width = winDims[0] + "px";
	
	var fixedHeight = 0;
	var scalingEls = [];
	
	if( this.elContent ){
		var els = this.elContent.getElementsByTagName("*");
		for( var i = 0; i < els.length; i++ ){
			var elChild = els[i];
			if( GIPROTOTYPE.hasClassName(elChild, "scaling-height") ){
				scalingEls.push(elChild);
			}else if( GIPROTOTYPE.hasClassName(elChild, "fixed-height") ){
				fixedHeight += GIPROTOTYPE.getDimensions(elChild, this._doc).height;
			}
		}
	}
	var scaledHeight = scalingEls.length==0? 0 : ((winDims[1] - fixedHeight) / scalingEls.length);
	for( var i = 0; i < scalingEls.length; i++ ){
		scalingEls[i].style.height = scaledHeight + "px";
	}	

}catch(e){
	if( GTDDEBUGMODE ) alert("GtdLightbox _position\n" + e.toString());
}
	
};


GtdLightbox.prototype.destroy = function(){
	this._eventObservers.destroy();
	this._eventHandlers.destroy();
	
	this._xmlHttpRequest = null;
	this._gtdinboxPage = null;
	this._doc = null;
	this._labels = null;
	
	this.el = null;
	this.elContainer = null;
};