/* Copyright Andy Mitchell 2007, 2008. No warranties */

function GtdPopuper(gtdinboxPage, el, assocObj){
	this.gtdinboxPage = gtdinboxPage;
	this.doc = gtdinboxPage.doc;
	this.window = gtdinboxPage.window;
	
	this.eventObservers = new GtdEventObservers(this);
	this.eventHandlers = new GtdEventHandlers_Mutator(this);
		
	this.el = el;
	if( assocObj ){
		this.assocObj = assocObj;
		this.assocObj.popup = GIPROTOTYPE.bind(this, this.popup);
		this.assocObj.poppedUp = GIPROTOTYPE.bind(this, this.poppedUp);
		this.assocObj.popupOff = GIPROTOTYPE.bind(this, this.popupOff);
		this.assocObj.positionPopup = GIPROTOTYPE.bind(this, this.positionPopup);
		this.assocObj.addPopupChild = GIPROTOTYPE.bind(this, this.addPopupChild);
		this.assocObj.removePopupChild = GIPROTOTYPE.bind(this, this.removePopupChild);
	}
	
	this.el.style.position = "absolute";
	this.el.style.display = "none";
	this.deadZones = [{el:this.el}];
	this.deadZoneFunctions = [];
}

GtdPopuper.prototype.positionPopup = function(type, options){
	// Align the popup against an element; either right or down.
	// Accept in form [x,y], an event (take pointer position), an element (take x/y)
try{	
	if( !this.on ) return;
	
	
	var location = (options && options.location )? options.location : "right";
	var failLocation = (options && options.failLocation )? options.failLocation : "left";
	var margin = (options && (options.margin || options.margin==0) )? options.margin : 0;
	
	var x = 0; var y = 0;
	var fBoundFail = null;
	if( type ){
		if( (type.currentTarget || type.target || type.srcElement) && !type.nodeName ){	// Event
			GIPROTOTYPE.stop(type);
			x = GIPROTOTYPE.pointerX(type);
			y = GIPROTOTYPE.pointerY(type);
			
			var popupDims = GIPROTOTYPE.getDimensions(this.el);
			switch(location){
				case "left":
					x = x - popupDims.width;
					if( x < 10 ) x = 10;
					break;
				case "bottomLeft":
					x = x - popupDims.width;
					y = y + margin;
					if( x < 10 ) x = 10;
					break;
			}
			
			fBoundFail = GIPROTOTYPE.bind(this, (function(xy){this.positionPopup([xy[0], y], {postFail:true})}));
		}else if( type instanceof Array ){
			x = type[0];
			y = type[1];
			fBoundFail = GIPROTOTYPE.bind(this, (function(xy){this.positionPopup([xy[0], y], {postFail:true})}));
		}else if( type.nodeName ){	// Element
			var el = type;
			if( el.nodeName=="A" ) el = el.parentNode;	// Cannot reference from A
			
			
			var borderWidth = el.parentNode? GIPROTOTYPE.getBorderWidth(el.parentNode) : 0;	// get width of el's parent (canvas?) border, which is not included in the cumulativeOffset
			var offset = GIPROTOTYPE.cumulativeOffset(el);
				offset[0] = offset[0] + borderWidth;
				offset[1] = offset[1] + borderWidth;
			var dims = GIPROTOTYPE.getDimensions(el);
			var popupDims = GIPROTOTYPE.getDimensions(this.el);


			switch(location){
				case "right":
					x = offset[0] + dims.width + margin;
					y = offset[1];
					break;
				case "left":
					x = offset[0] - dims.width - margin - popupDims.width;
					y = offset[1];
					break;
				case "bottomRight":
					x = offset[0] + dims.width + margin;
					y = offset[1] + dims.height + margin;
					break;
				case "bottomLeft":
					x = offset[0] - dims.width  - popupDims.width;
					y = offset[1] + dims.height + margin;
					break;
				case "bottom":
					x = offset[0];
					y = offset[1] + dims.height + margin;
					break;
				case "relative-bottom":
					x = 0;
					y = dims.height + margin;
					break;
				case "relative-bottom-bottom-right":
					x = dims.width - popupDims.width;
					y = dims.height + margin;
					break;
			}
			fBoundFail = GIPROTOTYPE.bind(this, (function(xy){this.positionPopup(type, {location:failLocation, margin:0, postFail:true})}));
		}
	}
	if( (x < 0) && location.indexOf("relative")==-1 ) x = 0;
	if( y < 0 ) y = 0;
	if(x) this.el.style.left = x + "px";
	if(y) this.el.style.top = y + "px";
	
	if(!(options && options.postFail)){
		var xy = null;
		if( xy = this.boundsCheck([x, y]) ){
			if( xy[0]!=0 && fBoundFail ){
				fBoundFail(xy);
			}
		}
	}
	
	this.eventHandlers.broadcast( "positionPopup" );
	
}catch(e){
	if( GTDDEBUGMODE ) alert("GtdPopuper positionPopup\n" + e.toString());
}

};
GtdPopuper.prototype.boundsCheck = function(offset){
	// See if we're in bounds; and if not, return array detailing the correction
		if( !offset || (offset[0]==0 && offset[1]==0) ){
			var display = this.el.style.display;
			this.el.style.display = "";
			offset = GIPROTOTYPE.cumulativeOffset(this.el);
			this.el.style.display = display;
		}
		var viewport = GIPROTOTYPE.viewport(this.doc, this.window, true);
		var dim = GIPROTOTYPE.getDimensions(this.el);

		var deltaX = 0; var deltaY = 0;
		if( (offset[0] + dim.width) > viewport[0] ){	// Move in
			deltaX = viewport[0] - (offset[0] + dim.width) -20;
		}
		if( (offset[1] + dim.height) > viewport[1] ){	// Move up
			deltaY = viewport[1] - (offset[1] + dim.height) - 20;
		}
		if( deltaX!=0 || deltaY!=0 ){
			return [deltaX?offset[0] + deltaX:0, deltaY?offset[1] + deltaY:0];
		}
};

GtdPopuper.prototype.popup = function(type, options){
	// Show the el at x/y; and activate mouse down listeners
try{
	this.on = true;
	
		
	this.positionPopup(type, options);
	if( this.el.style.display=="none" ){
		this.el.style.display = "";	
		this.el.style.zIndex = 500;
		if( options && options.popOffMouseout ){
			this.eventObservers.observe( this.gtdinboxPage.gmailDOM.element("document"), "mousemove", this.windowMouseMove, true );
		}else{
			this.eventObservers.observe( this.gtdinboxPage.gmailDOM.element("document"), "mousedown", this.windowMouseDown, true );
		}
		this.eventHandlers.broadcast("popup");
	}
}catch(e){
	if( GTDDEBUGMODE ) alert( "Popuper popup\n" + e.toString() );
}
};
GtdPopuper.prototype.poppedUp = function(){
	return (this.el.style.display!="none");
};

GtdPopuper.prototype.popupOff = function(){
	// Hide the popup element; and also any children it might have
	if( !this.on ) return;
	this.on = false;
	
	this.eventObservers.stopObserving( this.gtdinboxPage.gmailDOM.element("document"), "mousedown", this.windowMouseDown, true );
	this.eventObservers.stopObserving( this.gtdinboxPage.gmailDOM.element("document"), "mousemove", this.windowMouseMove, true );
	if( this.el.style.display!="none" ){
		this.el.style.display = "none";
		
		this.eventHandlers.broadcast("popupOff");
	}
	
	// Hide any children (and if that child 
	for( var i = 0; i < this.deadZones.length; i++ ){
		var child = this.deadZones[i];
		if( child.popuper ){	// It has popup, use that
			if( child.popuper.popupOff ) child.popuper.popupOff();
		}else if( child.style ){	// It's an element, just hide it
			child.style.display = "none";
		}
	}
};


GtdPopuper.prototype.addPopupChild = function(child){
	// Popup children will not close this popup if clicked; but will be hidden when this popup is hidden
	if( child.popup ){	// It is a popupable object
		this.deadZones[this.deadZones.length] = {'el':child.el, 'popuper':child};
	}else{	// It's just an element
		this.deadZones[this.deadZones.length] = {'el':child};
	}
};
GtdPopuper.prototype.removePopupChild = function(el){
	for( var i = 0; i < this.deadZones.length; i++ ) if( this.deadZones[i].el==el ) this.deadZones.splice(i, 1);
};
GtdPopuper.prototype.resetDeadZones = function(){
	this.deadZones.length = 1;
};

GtdPopuper.prototype.inhibitPopupOff = function(v){
	if( typeof v=="undefined" ) return this._inhibitPopupOff;
	this._inhibitPopupOff = v;
};

GtdPopuper.prototype.windowMouseDown = function(event){
	// Test bounds
	if( this._inhibitPopupOff ) return;
	if( !GIPROTOTYPE.within( this.el, GIPROTOTYPE.pointerX(event), GIPROTOTYPE.pointerY(event) ) ){
		if( !this.inPopupDeadzone(GIPROTOTYPE.element(event)) ) this.popupOff();
	}
};
GtdPopuper.prototype.windowMouseMove = function(event){
	// Test bounds
	if( !GIPROTOTYPE.within( this.el, GIPROTOTYPE.pointerX(event), GIPROTOTYPE.pointerY(event) ) ){
		if( !this.inPopupDeadzone(GIPROTOTYPE.element(event)) ){
			this.popupOff();
		}
	}
};
GtdPopuper.prototype.inPopupDeadzone = function(elTop){
	// Return true if el falls in a deadzone (include parents)
	for( var i = 0; i < this.deadZones.length; i++ ){	// Now scan up tree to see if this el is part of deadzone
		var el = elTop;
		while( el.parentNode ){
			if( el==this.deadZones[i].el ) return true;
			el = el.parentNode;
		}
	}
	
	// Now do function tests - that consumers of GtdPopuper can define
	for( var i = 0; i < this.deadZoneFunctions.length; i++ ){
		if( this.deadZoneFunctions[i](elTop) ) return true;
	}
}

GtdPopuper.prototype.addDeadZoneFunction = function(f){
	this.deadZoneFunctions.push(f);
};

GtdPopuper.prototype.destroy = function(){
	this.eventObservers.destroy();
	if( this.assocObj ){
		var ns = ["popup", "popupOff", "positionPopup", "addPopupChild", "removePopupChild"];
		for( var i = 0; i < ns.length; i++ ) if( this.assocObj[ns[i]] ) delete this.assocObj[ns[i]];
	}
};